// Copyright (C) Nial Peters 2011
//
// This file is part of blur.
// blur is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// blur is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with blur.  If not, see <http://www.gnu.org/licenses/>.

/*
 * sensor_interface.cpp
 *
 *  Created on: 10-Jun-2011
 *      Author: njp39
 */
#include"sensor_interface.h"


SensorInterface::SensorInterface(int serial_number)
{
			//initialise class attributes
			connected = false;
			this->serial_number = serial_number;

			if (CPhidgetInterfaceKit_create(&handle)==0){
				// note that if for some reason we fail to create the handle - we carry on
				// regardless - just means that none of the sensors will work

				//set callback functions for attachment and detachment of sensor
				CPhidget_set_OnAttach_Handler((CPhidgetHandle)handle, onAttach, this);
				CPhidget_set_OnDetach_Handler((CPhidgetHandle)handle, onDetach, this);

				//open the device - note that this returns immediately
				CPhidget_open((CPhidgetHandle)handle, serial_number);
				CPhidget_waitForAttachment((CPhidgetHandle)handle,1000);
				int open_status=-1;
				CPhidget_getDeviceStatus((CPhidgetHandle)handle, &open_status);

				if(open_status == PHIDGET_NOTATTACHED){
					WARN("Could not connect to sensor interface #%d - are you root?",serial_number);
				}
			}else{
				WARN("Could not create handle for sensor interface #%d",serial_number);
			}
}



SensorInterface::~SensorInterface(void){
	//close USB connections and shut down threads
	if (handle != NULL){
		CPhidget_close((CPhidgetHandle)handle);
		CPhidget_delete((CPhidgetHandle)handle);
	}
}



void SensorInterface::printStats(void){
	int ratiometric;
	if (! connected){
		std::cout << "SensorInterface (Serial #"<<serial_number<<"): Not connected.\n";
	}else{
		std::cout << "SensorInterface (Serial #"<<serial_number<<"): Connected.\n";
		CPhidgetInterfaceKit_getRatiometric(handle, &ratiometric);
		std::cout << "Ratiometric = "<<ratiometric<<"\n";
	}

}



int SensorInterface::getRawSensorValue(int index){
	int raw_reading = -1;

	if(connected){
		CPhidgetInterfaceKit_getSensorRawValue(handle, index, &raw_reading);
	}
	return raw_reading;
}



bool SensorInterface::isAttached(void){
	return connected;
}



void SensorInterface::initialise(void){
	connected = true;

	CPhidgetInterfaceKit_setRatiometric(handle, 1);
}



void SensorInterface::disconnected(void){
	connected = false;
}



int onAttach(CPhidgetHandle h, void *userptr){
	//initialise the interface to have the required settings
	DEBUG("Sensor interface attached");
	((SensorInterface*)userptr)->initialise();
	return 0;
}



int onDetach(CPhidgetHandle h, void *userptr){
	DEBUG("Sensor interface detached");
	((SensorInterface*)userptr)->disconnected();
	return 0;
}
