// Copyright (C) Nial Peters 2011
//
// This file is part of blur.
// blur is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// blur is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with blur.  If not, see <http://www.gnu.org/licenses/>.

/*
 * sensor.cpp
 *
 *  Created on: 09-Jun-2011
 *      Author: njp39
 */
#include"sensors.h"


AnalogSensor::AnalogSensor(SensorInterface& interface, int index, const char *name="no name", double correction=0.0):
	interface(interface)
{
	//initialise class attributes
	this->name = std::string(name);
	this->index = index;
	this->correction = correction;
}


bool AnalogSensor::isWorking(void){
	if (readValue() == SENSOR_ERROR){
		return false;
	}
	return true;
}



double TemperatureSensor::readValue(){
	int raw_value = interface.getRawSensorValue(index);

	if (raw_value == -1){
		//interface wasn't connected
		return SENSOR_ERROR;
	}

	//conversion factors taken from Phidgets manual.
	return (0.054261294261294266 * raw_value) - 61.111 + correction;

}



double CurrentSensor::readValue(){
	int raw_value = interface.getRawSensorValue(index);

	if (raw_value == -1){
		//interface wasn't connected
		return SENSOR_ERROR;
	}

	//conversion factors taken from Phidgets manual.
	return (0.018500018500018502 * raw_value) - 37.8787 + correction;

}



double VoltageSensor::readValue(){
	int raw_value = interface.getRawSensorValue(index);

	if (raw_value == -1){
		//interface wasn't connected
		return SENSOR_ERROR;
	}

	//conversion factors taken from Phidgets manual.
	return ((raw_value/819.0)-2.5)/0.0681 + correction;

}
