// Copyright (C) Nial Peters 2011
//
// This file is part of blur.
// blur is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// blur is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with blur.  If not, see <http://www.gnu.org/licenses/>.

/*
 * sensor.hpp
 *
 *  Created on: 09-Jun-2011
 *      Author: njp39
 */

#ifndef SENSOR_H_
#define SENSOR_H_

#include<stdio.h>
#include<iostream>
#include<string>
#include<phidget21.h>
#include"sensor_interface.h"

//define value that will be returned when we can't read from the sensor
#define SENSOR_ERROR -1000.0

class AnalogSensor
{
	public:
		AnalogSensor(SensorInterface &interface, int index, const char *name, double correction);

		virtual double readValue(void)=0;
		bool isWorking(void);

	protected:
		std::string name;
		int index;
		double correction;
		SensorInterface &interface;
};



class TemperatureSensor : public AnalogSensor
{
	public:
		TemperatureSensor(SensorInterface &interface, int input, const char *name="Unnamed temperature sensor", double correction=0.0) : AnalogSensor(interface, input, name, correction){};
		double readValue(void);
};



class CurrentSensor : public AnalogSensor
{
	public:
		CurrentSensor(SensorInterface &interface, int input, const char *name="Unnamed current sensor", double correction=0.0) : AnalogSensor(interface, input, name, correction){};
		double readValue(void);
};



class VoltageSensor : public AnalogSensor
{
	public:
		VoltageSensor(SensorInterface &interface, int input, const char *name="Unnamed voltage sensor", double correction=0.0) : AnalogSensor(interface, input, name, correction){};
		double readValue(void);
};


#endif /* SENSOR_H_ */
