// Copyright (C) Nial Peters 2011
//
// This file is part of blur.
// blur is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// blur is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with blur.  If not, see <http://www.gnu.org/licenses/>.

/*
 * str_functions.cpp
 *
 *  Created on: 3 Nov 2011
 *      Author: nialp
 */
#include"str_functions.h"

int string_replace(char* output, size_t max_len, const char *str, const char *substr, const char *replacement_substr){
	/*
	 * Replaces all occurences of substr in str with substr_replacement. max_len specifies
	 * the maximum length that can be written to output.
	 *
	 * Returns number of replacements made, or -1 on failure.
	 */
	const char *s,*e;
	int i=0;
	size_t substr_len = strlen(substr);
	size_t replacement_substr_len = strlen(replacement_substr);

	if(strlen(str) > max_len){
		return -1;
	}

	s = str;
	e = strstr(str, substr);


	while (e != NULL){
		i++;
		if((e-s+replacement_substr_len)>max_len){
			return -1;
		}

		strncpy(output, s, e-s);
		output += (e-s);
		strcpy(output, replacement_substr);
		output += replacement_substr_len;
		s = e + substr_len;
		e = strstr(s, substr);
	}

	strcpy(output, s);
	return i;

}



int string_ends_with(const char* str, const char* ending){
	//
	// Returns 1 if str ends with ending, 0 otherwise
	//

	//retrieve pointer to expected location of ending of str
	size_t str_len = strlen(str);
	size_t ending_len = strlen(ending);
	const char * expected_pos = str + (str_len-ending_len);
	
	if (strcmp(expected_pos, ending)){
		return 0;
	}
	return 1;
}

size_t stringFromTime(char *s, size_t max_s, const char *fmt, time_t seconds, int milliseconds){
	//The problem here is that C/C++ doesn't really have a strftime function that
	//can handle milliseconds (maybe there is something in Boost, but I couldn't
	//find it). This is very tedious! So, this function works by first replacing the
	//format specifier for milliseconds (%s seemed like a good choice) with the actual
	//number of milliseconds using a custom written replace function, then passing that
	//format string to the built-in strftime function.

	//sanity check
	if(milliseconds > 999){
		//FATAL("Timestamp has more than 1000 milliseconds in it!");
	}

	char new_fmt[max_s];
	char ms_str[4];

	//convert milliseconds to a string
	snprintf(ms_str, 4, "%03d", milliseconds);

	//do the millisecond format specifier replacement
	if (string_replace(new_fmt, 196, fmt, "%s",ms_str) < 0){
		return -1;
	}

	//now call strftime on the new format string
	struct tm *tp = gmtime(&seconds);
	return strftime(s, max_s, new_fmt, tp);
}
