// Copyright (C) Nial Peters 2011
//
// This file is part of blur.
// blur is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// blur is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with blur.  If not, see <http://www.gnu.org/licenses/>.


/*
 * camera.hpp
 *
 *  Created on: 30 Oct 2011
 *      Author: nialp
 */

#ifndef CAMERA_HPP_
#define CAMERA_HPP_

//do the aravis imports (provides the GeniCam interface)
extern "C" {
#include <arv.h>
}
#include <glib-object.h>

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <string>
#include <sstream>
#include "logging.h"
#include "image.h"

class Image;

class CameraBase
{
	public:
		virtual ~CameraBase(void);
		virtual ImageBase* captureImage(void)=0;
		int getConfig(const char* name, bool &val);
		int getConfig(const char* name, long &val);
		int getConfig(const char* name, float &val);
		int getConfig(const char* name, char** val);

		int setConfig(const char* name, bool val);
		int setConfig(const char* name, long val);
		int setConfig(const char* name, double val);
		int setConfig(const char* name, const char *val);

		static void onControlLost(void *x);

	protected:
		CameraBase(void);
		virtual int initCamera();
		ArvDevice *device;
		ArvStream *stream;
		ArvGc *genicam;

};


class SimpleCamera: public CameraBase
{
	public:
		static SimpleCamera *createCamera(bool enable_fast_compress);
		ImageBase * captureImage(void);
		void recycleImage(ImageBase *im);
		~SimpleCamera(void);

	protected:
		SimpleCamera(void){buffer_flag=false;};
		virtual int initCamera(ImageBase *im);
		int reInitialise(void);

	private:
		bool buffer_flag;
		ArvGcNode *capture_node;
		ImageBase *image;
};


#endif /* CAMERA_HPP_ */
