// Copyright (C) Nial Peters 2011
//
// This file is part of blur.
// blur is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// blur is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with blur.  If not, see <http://www.gnu.org/licenses/>.


//code specific to the FLIR SC645 camera
#include <libconfig.h++>
#include <sstream>
#include <math.h>
#include <limits.h>
#include <stdint.h>

#include "camera.h"
#include "image.h"
using namespace libconfig;

class IRImage : public ImageBase{
	public:
		IRImage(bool enable_fast_compress, int low_T_threshold): ImageBase(enable_fast_compress){ 
									this->low_T_threshold = low_T_threshold;};
	protected:
		int preProcess(void);

	private:
		int low_T_threshold;
		void getMinMax(uint16_t &min_pix, uint16_t &max_pix);
		void convertTo8bit(int min_pix, int max_pix);
};


class SC645Camera : public SimpleCamera{

	public:
		static SC645Camera * createCamera(Setting &camera_cfg, Setting &capture_cfg);
		IRImage * captureImage(void);
		void recycleImage(IRImage *im){this->SimpleCamera::recycleImage((ImageBase *)im);};
		int setFocusPos(int focus_pos);
		int refocus(void);
		int autofocus(void);
		int performNUC(void);

	protected:
		int initCamera(IRImage *im,Setting &camera_cfg);

	private:
		IRImage *image;
		int focus_position;

};


