from distutils.core import setup,Extension
import sys
import os.path

required_modules = [("numpy","Numpy"),
                    ("Image", "the Python Imaging Library"),
                    ("wx", "wxPython"),
                    ("matplotlib", "Matplotlib")
                    ]

#check that all the required modules are available
print "Checking for required Python modules..."
for mod,name in required_modules:
    try:
        print "importing "+mod+"..."
        __import__(mod)
    except ImportError:
        print "Failed to import \'"+mod+"\'. Please ensure that "+name+" is correctly installed, then re-run this installer."
        sys.exit(1)


if sys.platform == "linux2":
    apps_folder = '/usr/share/applications'
    icon_folder = '/usr/share/icons'
    desktop_folder = os.path.expandvars("$HOME/Desktop")
    
    files_to_install = [(icon_folder,['icons/blur_viewer.png']),
                        (apps_folder,['misc/blur-view.desktop']),
                        (desktop_folder,['misc/blur-view.desktop'])]
    scripts_to_install = [os.path.normpath('src/blur_view.py')]
    
elif sys.platform == "win32":
    files_to_install = [("icons",[os.path.join('icons','blur_viewer.ico')])]
    scripts_to_install = [os.path.join('src','blur_view.py'), os.path.join('misc','blur_view_win32_postinstall.py')]
    
else:
    raise RunTimeError, "Unsupported platform"



print "Running setup..."
setup(name='blur_view',
      version='1.0',
      description='Real-time image viewer for the blur image capture program.', 
      author = 'Nial Peters', 
      author_email='nonbiostudent@hotmail.com',
      url='',
      scripts=scripts_to_install,
      data_files=files_to_install)

#post install
if sys.argv.count('install') != 0:
    if sys.platform == "linux2":
        try:
        	you = os.getlogin()
        except OSError:
        	you = os.path.expandvars("$LOGNAME")
        #TODO - this will return root!	
        if(you == "$LOGNAME"):
        	raise RuntimeError, "Failed to determine your login name"
        #make sure the .desktop file on the desktop is executable
        for f in [desktop_folder+'/blur-view.desktop']:
            return_code = os.system("chmod a+x "+f)
            if return_code != 0:
                print "Error! Failed to change execute permissions on \'"+f+"\'"
        
        os.system("chown "+you+":"+you+" "+desktop_folder+'/blur-view.desktop')
            
            
            
