#!/usr/bin/python
import socket
import Queue
import threading
import Image
import StringIO
import time
import Image
import sys
import wx
import os.path
import re
import cPickle
import numpy
import socket
import threading
import struct

import matplotlib
matplotlib.use('WXAgg') #needs to come straight after matplotlib import

from matplotlib import cm
from matplotlib.backends.backend_wxagg import FigureCanvasWxAgg
from matplotlib.backends.backend_wx import NavigationToolbar2Wx
from matplotlib.figure import Figure
from matplotlib.colors import ListedColormap

class SensorDataReceiver:
    def __init__(self, ip_addr, port):
        
        self.ip_addr = ip_addr
        self.port = port
        self._stay_alive = True
        self._sock = None
        self.__data_queue = Queue.Queue()
        
        self._worker_thread = threading.Thread(target=self._receive_sensor_data)
        self._worker_thread.start()
    
    
    def stop(self):
        
        self._stay_alive = False
        if self._sock is not None:
            self._sock.shutdown(socket.SHUT_WR)
        self._worker_thread.join()
    
    
    def get(self, timeout=10):
        return self.__data_queue.get(timeout=timeout)
        
            
    def _receive_sensor_data(self):
        while self._stay_alive:           
            self._sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            
            try:
                self._sock.connect((self.ip_addr, int(self.port)))
            except socket.error:
                self._sock = None
                break
           
            data = ''
            n=1
            while (n>0):
                n = self._sock.recv(1000000)
                if n=='':
                    break
                data += n
            
            if data == '':
                break
            #unpack the string into values
            if len(data) == struct.calcsize("dddd"):
            
                sensor_values = struct.unpack("dddd",data)
                self.__data_queue.put({'voltage':sensor_values[0],'current':sensor_values[1],
                                       'internal_T':sensor_values[2], 'external_T':sensor_values[3]})
            else:
                print "Received data had incorrect length"
            
            self._sock.close()
            self._sock = None
        
        self.__data_queue.put(None)



class ImageReceiver:
    def __init__(self, ip_addr, port, image_save_dir, remote_save_dir):
        
        self.ip_addr = ip_addr
        self.port = port
        self.image_save_dir = image_save_dir
        self.remote_save_dir = remote_save_dir
        self._stay_alive = True
        self._sock = None
        self.__image_queue = Queue.Queue()
        self.__remote_save_queue = Queue.Queue()
        
        self._worker_thread = threading.Thread(target=self._receive_images)
        self._worker_thread.start()
        
        if remote_save_dir is not None:
            self._remote_save_thread = threading.Thread(target=self._save_remotely)
            self._remote_save_thread.start()
        else:
            self._remote_save_thread = None
        
    
    def _save_remotely(self):
        while self._stay_alive:
            im_data = self.__remote_save_queue.get()
            
            if im_data is None:
                #exit request
                break
            
            #now pull all the data out of the queue (only want the last image)
            try:
                while True:
                    im_data = self.__remote_save_queue.get_nowait()
            except Queue.Empty:
                if im_data is None:
                    break
                
                try:
                    self._save_image(im_data[0], im_data[1], self.remote_save_dir)
                except:
                    pass
        
            
    def stop(self):
        
        self._stay_alive = False
        if self._sock is not None:
            self._sock.shutdown(socket.SHUT_WR)
            
        if self._remote_save_thread is not None:
            self.__remote_save_queue.put(None)
        self._worker_thread.join()
        self._remote_save_thread.join()
    
    
    def get(self, timeout=10):
        return self.__image_queue.get(timeout=timeout)
        
            
    def _receive_images(self):
        while self._stay_alive:           
            self._sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            
            try:
                self._sock.connect((self.ip_addr, int(self.port)))
            except socket.error:
                self._sock = None
                break
           
            data = ''
            n=1
            while (n>0):
                n = self._sock.recv(1000000)
                if n=='':
                    break
                data += n
            
            if data == '':
                break
            
            ss = StringIO.StringIO(data)
            try:
                image = Image.open(ss)
            except:
                image = Image.new('L',(340,480),color=0)
            
            if self.image_save_dir is not None:
                self._save_image(data, float(image.info['capture_time']), self.image_save_dir)
            
            if self.remote_save_dir is not None:
                self.__remote_save_queue.put((data, float(image.info['capture_time'])))
            
            self.__image_queue.put(image)
            self._sock.close()
            self._sock = None
        
        self.__image_queue.put(None)
    
    
    def _save_image(self, data, cap_time, path):
        #saves raw image data to file (not an Image object - no point in 
        # having to re-compress the data since it is already a png)
        cap_time = datetime.datetime.utcfromtimestamp(cap_time)
        
        im_filename = os.path.normpath(cap_time.strftime("%Y/%d%b/%H/%H%M%S.%f.png"))
        im_path = os.path.join(path, im_filename)
        
        try:
            os.makedirs(os.path.dirname(im_path))
        except OSError:
            pass #dir already exists
        
        with open(im_path, 'wb') as ofp:
            ofp.write(data)
        
        
        
class WaitingForConnectionDialog(wx.Dialog):
    
    def __init__(self, parent, ip, port):

        # Call wxDialog's __init__ method
        wx.Dialog.__init__ (self, parent, -1, "blur_view", size = (200,200))
        
        # Create a panel for our dialog
        self.panel = wx.Panel ( self, -1 )
        
        # Create a label
        self.label = wx.StaticText ( self.panel, -1, "Trying to connect to \'"+ip+":"+str(port)+"\'")
        
        self.vsizer = wx.BoxSizer ( wx.VERTICAL )
        self.hsizer = wx.BoxSizer ( wx.HORIZONTAL )
        self.hsizer.AddSpacer(20)
        art = wx.ArtProvider()
        icon = wx.StaticBitmap(self.panel, bitmap=art.GetBitmap(wx.ART_INFORMATION, client=wx.ART_MESSAGE_BOX))
        self.hsizer.Add(icon,0,wx.ALIGN_LEFT) 
        self.hsizer.AddSpacer(5)
        self.hsizer.Add ( self.label, 0, wx.ALIGN_CENTER )
        self.hsizer.AddSpacer(20)
        self.vsizer.AddSpacer(20)
        self.vsizer.Add ( self.hsizer, 0, wx.ALIGN_CENTER )
        self.vsizer.AddSpacer(10)
        self.cancel_button = wx.Button(self.panel, wx.ID_ANY, "Cancel")
        self.vsizer.Add (self.cancel_button, 0 , wx.ALIGN_CENTRE_HORIZONTAL)
        self.vsizer.AddSpacer(20)
        # Attach the sizer to the panel
        self.panel.SetSizerAndFit ( self.vsizer )
        
        # Resize the dialog
        self.vsizer.Fit ( self )
        
        wx.EVT_BUTTON(self,self.cancel_button.GetId(), self.onCancel)
        wx.EVT_CLOSE(self, self.onCancel)
        
    def onCancel(self, evnt):
        self.EndModal(wx.ID_CANCEL)
    

       
class MainFrame(wx.Frame):
    def __init__(self, ip, port, image_save_dir, remote_image_dir):
        self.im = None
        self.created = False
        self.warn_txt = None
        self.im_receiver = None
        self.sensor_receiver = None
        self.ip = ip
        self.port = port
        self.image_save_dir = image_save_dir
        self.remote_image_dir = remote_image_dir
        wx.Frame.__init__(self, None, -1, "blur_view")
        self._stay_alive=True
        top_panel = wx.Panel ( self, -1 )
        self.fig = Figure((5,4), 75)
        self.canvas = FigureCanvasWxAgg(top_panel, -1, self.fig)
        self.toolbar = NavigationToolbar2Wx(self.canvas)
        self.toolbar.Realize()
        
        self.status_bar = self.CreateStatusBar()
               
        # Now put all into a sizer
        sizer = wx.BoxSizer(wx.VERTICAL)

        # This way of adding to sizer allows resizing
        sizer.Add(self.canvas, 1, wx.ALIGN_LEFT|wx.ALIGN_TOP|wx.EXPAND)
        
        # Best to allow the toolbar to resize!
        sizer.Add(self.toolbar, 0, wx.EXPAND)
        top_panel.SetSizer(sizer)
        sizer.Fit(self)
        top_panel.SetAutoLayout(True)
        self.connecting_dialog = WaitingForConnectionDialog(self, ip, port)     
        self._worker_thread = threading.Thread(target=self.__run_img_recv)
        self._worker_thread.start()
        self._sensor_thread = threading.Thread(target=self.__run_sensor_recv)
        self._sensor_thread.start()
        
        wx.EVT_CLOSE(self,self.exit)
           
        if self.connecting_dialog.ShowModal() == wx.ID_CANCEL:
            self.exit()
            
             
        self.Show(True)
        self.created = True

    
    def __run_sensor_recv(self):
        self.sensor_receiver = SensorDataReceiver(self.ip, str(int(self.port) + 1)) #TODO - +1 to the port is a hack - it might not always be this
        #wait for the initial connection
        while self._stay_alive:
            try:
                data = self.sensor_receiver.get(timeout=1)
                self.first_sensor_data = True
                break
            except Queue.Empty:
                continue
        
        if not self._stay_alive:
            return
        
        if data is None:
            time.sleep(2)
            self.__run_sensor_recv()
            return
        
        wx.CallAfter(self._on_new_sensor_data, data)
        
        while self._stay_alive:
            try:
                data = self.sensor_receiver.get(timeout=15)
                
                if data is None:
                    return
                                
                wx.CallAfter(self._on_new_sensor_data, data)
            except Queue.Empty:
                pass
    
    def __run_img_recv(self):
        
        self.im_receiver = ImageReceiver(self.ip, self.port, self.image_save_dir, self.remote_image_dir)
        
        #wait for the initial connection
        while self._stay_alive:
            try:
                im = self.im_receiver.get(timeout=0.5)
                self.first_im=True
                break
            except Queue.Empty:
                continue
        
        if not self._stay_alive:
            return
        
        if im is None:
            time.sleep(2)
            self.__run_img_recv()
            return
        
        wx.CallAfter(self.connecting_dialog.EndModal, wx.ID_OK)
        
        wx.CallAfter(self._on_new_image, im)
        
        while self._stay_alive:
            try:
                im = self.im_receiver.get(timeout=15)
                
                if im is None:
                    if self._stay_alive:
                        wx.CallAfter(self.onServerDisconnect)
                    return
                
                wx.CallAfter(self._on_new_image, im)
            except Queue.Empty:
                wx.CallAfter(self.on_blur_serv_timeout)
        
    
    def exit(self, *args):
        self.Show(False)
        wx.Yield()
        self._stay_alive = False
        if self.im_receiver is not None:
            self.im_receiver.stop()
        if self.sensor_receiver is not None:
            self.sensor_receiver.stop()
        self._worker_thread.join()
        self._sensor_thread.join()
        self.Destroy()
        sys.exit()

    
    def onServerDisconnect(self):
        self.Show(False)
        self._stay_alive = False
        if self.im_receiver is not None:
            self.im_receiver.stop()
        if self.sensor_receiver is not None:
            self.sensor_receiver.stop()
        self._worker_thread.join()
        self._sensor_thread.join()
        
        self.connecting_dialog = WaitingForConnectionDialog(self, self.ip, self.port)
        self.connecting_dialog.CenterOnScreen()
        
        self._worker_thread = threading.Thread(target=self.__run_img_recv)
        self._worker_thread.start()
        self._sensor_thread = threading.Thread(target=self.__run_sensor_recv)
        self._sensor_thread.start()
        
        if self.connecting_dialog.ShowModal() == wx.ID_CANCEL:
            self.exit()

    
        
    def on_blur_serv_timeout(self):
        
        if self.warn_txt is not None:
            return
        a = self.im.get_axes()
        self.warn_txt =  a.text(0.5, 0.5,
                                'ERROR!\nTimeout reached!', 
                                transform = a.transAxes,
                                horizontalalignment='center',
                                verticalalignment='center',
                                fontsize=50,
                                color='r')
        self.canvas.draw()
        wx.YieldIfNeeded()
        self.canvas.gui_repaint()
        wx.YieldIfNeeded()

    
    def _on_new_sensor_data(self, data):
        wx.CallAfter(self.status_bar.SetStatusText,"Voltage: "+str(round(data['voltage'],1))+
                                      "\tCurrent: "+str(round(data['current'],2))+
                                      "\tInternal Temp.: "+str(round(data['internal_T'],1))+
                                      "\tExternal Temp.: "+str(round(data['external_T'],1)))
    
    def _on_new_image(self, im):
        if self.warn_txt is not None:
            self.warn_txt.remove()
            self.warn_txt = None
        if self.first_im:
            a = self.fig.add_axes((0,0,1,1))
            wx.YieldIfNeeded()
            self.im = a.imshow(numpy.asarray(im), origin='upper', cmap=iron_cmap)
            a.set_xticks([])
            a.set_yticks([])
            self.first_im=False
        else:
            self.im.set_array(numpy.asarray(im))

        self.canvas.draw()
        wx.YieldIfNeeded()
        self.canvas.gui_repaint()
        wx.YieldIfNeeded()


class ConnectionSettingsFrame(wx.Frame):
    
    def __init__(self):
        
        #attempt to load the last settings from the persistence file
        persistence_file = os.path.expanduser("~/.blur_view_persist")
        if os.path.exists(persistence_file):
            with open(persistence_file,"rb") as ifp:
                default_settings = cPickle.load(ifp)
        else:
            default_settings = ('127.0.0.1', '24601', "","")
        
        if len(default_settings) != 4:
            default_settings = ('127.0.0.1', '24601', "","")
        
        wx.Frame.__init__(self, None, wx.ID_ANY, "Connection Settings - blur_view")
        
        top_panel = wx.Panel(self, wx.ID_ANY)
        
        vsizer = wx.BoxSizer(wx.VERTICAL)
        
        ip_sizer = wx.BoxSizer(wx.HORIZONTAL)
        port_sizer = wx.BoxSizer(wx.HORIZONTAL)
        save_sizer = wx.BoxSizer(wx.HORIZONTAL)
        remote_save_sizer = wx.BoxSizer(wx.HORIZONTAL)
        
        ip_text = wx.StaticText(top_panel, wx.ID_ANY, "IP:")
        self.ip_box = wx.TextCtrl(top_panel, size=(200,25))
        self.ip_box.SetValue(default_settings[0])    
        
        ip_sizer.Add(ip_text, 0, wx.ALIGN_RIGHT|wx.ALIGN_CENTRE_VERTICAL)
        ip_sizer.Add(self.ip_box, 0, wx.ALIGN_RIGHT|wx.ALIGN_CENTRE_VERTICAL)
               
        port_text = wx.StaticText(top_panel, wx.ID_ANY, "Port:")
        self.port_box = wx.TextCtrl(top_panel, size=(80,25))
        self.port_box.SetValue(default_settings[1])       
        port_sizer.Add(port_text, 0, wx.ALIGN_RIGHT|wx.ALIGN_CENTRE_VERTICAL)
        port_sizer.Add(self.port_box, 0,wx.ALIGN_RIGHT|wx.ALIGN_CENTRE_VERTICAL)
               
        save_dir_txt = wx.StaticText(top_panel, wx.ID_ANY, "Save to local directory:")
        self.save_dir_box = wx.TextCtrl(top_panel, size=(200,25))
        self.save_dir_box.SetValue(default_settings[2])
        self.browse_button = wx.Button(top_panel, wx.ID_ANY, "Browse")       
        
        save_sizer.Add(self.save_dir_box, 0,wx.ALIGN_RIGHT|wx.ALIGN_CENTRE_VERTICAL)
        save_sizer.Add(self.browse_button,0,wx.ALIGN_RIGHT|wx.ALIGN_CENTRE_VERTICAL)
        
        remote_save_dir_txt = wx.StaticText(top_panel, wx.ID_ANY, "Save to remote directory:")
        self.remote_save_dir_box = wx.TextCtrl(top_panel, size=(200,25))
        self.remote_save_dir_box.SetValue(default_settings[3])
        self.remote_browse_button = wx.Button(top_panel, wx.ID_ANY, "Browse")       
        
        remote_save_sizer.Add(self.remote_save_dir_box, 0,wx.ALIGN_RIGHT|wx.ALIGN_CENTRE_VERTICAL)
        remote_save_sizer.Add(self.remote_browse_button,0,wx.ALIGN_RIGHT|wx.ALIGN_CENTRE_VERTICAL)        
        
        
        button_sizer = wx.BoxSizer(wx.HORIZONTAL)
        self.ok_button = wx.Button(top_panel, wx.ID_ANY, "Connect")
        self.cancel_button = wx.Button(top_panel, wx.ID_ANY, "Cancel")
        button_sizer.Add(self.cancel_button,0,wx.ALIGN_RIGHT)
        button_sizer.Add(self.ok_button,0,wx.ALIGN_RIGHT)
        button_sizer.AddSpacer(10)
        
        wx.EVT_BUTTON(self, self.browse_button.GetId(), self.onBrowse)
        wx.EVT_BUTTON(self, self.remote_browse_button.GetId(), self.onRemoteBrowse)
        wx.EVT_BUTTON(self, self.ok_button.GetId(), self.onOk)
        wx.EVT_BUTTON(self, self.cancel_button.GetId(), self.onCancel)
        wx.EVT_CLOSE(self, self.onCancel)
        
        vsizer.AddSpacer(10)
        vsizer.Add(ip_sizer,0, wx.ALIGN_CENTER_HORIZONTAL)
        vsizer.Add(port_sizer,0, wx.ALIGN_CENTER_HORIZONTAL)
        vsizer.AddSpacer(10)
        vsizer.Add(save_dir_txt,0,wx.ALIGN_LEFT)
        vsizer.Add(save_sizer, 0, wx.ALIGN_CENTER_HORIZONTAL)   
        vsizer.AddSpacer(10)
        vsizer.Add(remote_save_dir_txt,0,wx.ALIGN_LEFT)
        vsizer.Add(remote_save_sizer, 0, wx.ALIGN_CENTER_HORIZONTAL) 
        vsizer.AddSpacer(10)    
        vsizer.Add(button_sizer,1, wx.ALIGN_BOTTOM|wx.ALIGN_RIGHT)
        vsizer.AddSpacer(10)
        
        hsizer = wx.BoxSizer(wx.HORIZONTAL)
        hsizer.AddSpacer(50)
        hsizer.Add(vsizer,1,wx.ALIGN_CENTER_HORIZONTAL)
        hsizer.AddSpacer(50)
        
        top_panel.SetSizer(hsizer)
        hsizer.Fit(self)
        top_panel.SetAutoLayout(1)
        self.CenterOnScreen()
        self.Show()

        
    def onRemoteBrowse(self,evnt):
        dialog = wx.DirDialog(self, "Choose remote image output directory")
        if dialog.ShowModal() != wx.ID_CANCEL:
            self.remote_save_dir_box.SetValue(dialog.GetPath()) 
     
    def onBrowse(self,evnt):
        dialog = wx.DirDialog(self, "Choose image output directory")
        if dialog.ShowModal() != wx.ID_CANCEL:
            self.save_dir_box.SetValue(dialog.GetPath())
    
    def onOk(self, evnt):
        ip_regex = r'(?P<ip>(?:[0-9]{1,3}\.){3}[0-9]{1,3}(?!.))'
        
        ip_str = self.ip_box.GetValue().lstrip().rstrip()
        port_str = self.port_box.GetValue().lstrip().rstrip()
        
        m = re.match(ip_regex, ip_str)
        if m is None:
            wx.MessageBox("Invalid IP address.","blur_view" ,wx.ICON_ERROR)
            return
        ip_address = m.group('ip')
        
        try:
            port = int(port_str)
        except ValueError:
            wx.MessageBox("Invalid port number.","blur_view", wx.ICON_ERROR)
            return
                        
        save_dir = self.save_dir_box.GetValue()
        remote_save_dir = self.remote_save_dir_box.GetValue()

        #attempt to store the values in the persistent file
        persistence_file = os.path.expanduser("~/.blur_view_persist")
        try:
            ofp = open(persistence_file,"wb")
            cPickle.dump((ip_address, port_str, save_dir, remote_save_dir), ofp)
            ofp.close()
        except:
            pass
               
        if save_dir ==  "":
            save_dir = None
        
        if remote_save_dir == "":
            remote_save_dir = None
            
        #otherwise launch the main frame
        self.Show(False)
        m = MainFrame(ip_address, port_str,save_dir, remote_save_dir)
        self.Destroy()
        
        
    def onCancel(self, evnt):
        self.Destroy()
        
        
if __name__ == '__main__':
    from pylab import *
    
    try:
        a = numpy.loadtxt(os.path.expandvars('${HOME}/iron.cmap'))
        iron_cmap = ListedColormap(a/255.0,name='iron')
    except:
        iron_cmap = cm.gray
    
    app = wx.PySimpleApp()
    f = ConnectionSettingsFrame()
    app.MainLoop()
